SystemOrganization addCategory: #'GT-BytecodeDebugger'!

!ChangeDebuggerAction class methodsFor: '*GT-BytecodeDebugger' stamp: 'AndreiChis 10/22/2014 15:16'!
gtBytecodeDebuggerActionFor: aDebugger
	<gtBytecodeStackDebuggingAction>
	
	^ self gtChangeDebuggerActionsFor: aDebugger! !

!SymbolicBytecode methodsFor: '*GT-BytecodeDebugger' stamp: 'AndreiChis 1/14/2015 17:10'!
= anObject

	self == anObject
		ifTrue: [^ true].

	(self class == anObject class)
		ifFalse: [^ false].

	^ (self bytes = anObject bytes) and: [ 
		(self description = anObject description) and: [ 
			(self method = anObject method) and: [ 
				self offset = anObject offset ] ] ]! !

DebugSession subclass: #GTBytecodeDebuggerSession
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-BytecodeDebugger'!

!GTBytecodeDebuggerSession commentStamp: 'AndreiChis 10/22/2014 15:03' prior: 0!
A GTBytecodeDebuggerSession provides debugging actions at the level of the bytecode
!

!GTBytecodeDebuggerSession methodsFor: 'debugging actions' stamp: 'AndreiChis 10/22/2014 14:28'!
stepIntoBytecodeInContext: aContext

	(self isContextPostMortem: aContext) ifTrue: [^ self].

	interruptedProcess step: aContext.
	self updateContextTo:  interruptedProcess suspendedContext.! !

!GTBytecodeDebuggerSession methodsFor: 'debugging actions' stamp: 'AndreiChis 10/22/2014 14:30'!
stepOverBytecodeInContext: aContext
	| newContext |
	
	(self isContextPostMortem: aContext) ifTrue: [^ self].

	newContext := interruptedProcess completeStep: aContext.
	self updateContextTo:  
		(newContext == aContext
			ifTrue: [ interruptedProcess suspendedContext ]
			ifFalse: [ newContext ]).! !

!GTBytecodeDebuggerSession methodsFor: 'acessing' stamp: 'AndreiChis 10/22/2014 14:47'!
symbolicBytecodeForCurrentPCFrom: aContext
	| pc |
	
	pc := aContext pc.
	^ aContext method symbolicBytecodes 
		detect: [ :each | each offset = pc ]
		ifNone: [ nil ]! !

!GTBytecodeDebuggerSession methodsFor: 'evaluating' stamp: 'AndreiChis 10/22/2014 15:19'!
unwindAndRestartToContext: aContext
	|ctx|

	ctx := interruptedProcess popTo: aContext.
	ctx == aContext ifTrue: [ 
		"Only restart the process if the stack was unwind"
		interruptedProcess
			restartTop;
			suspendedContext ].
	self flag: 'Should a warning be displayed if the the unwind failed?'.
	self updateContextTo:  aContext! !

GTGenericStackDebugger subclass: #GTBytecodeDebugger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-BytecodeDebugger'!

!GTBytecodeDebugger commentStamp: 'AndreiChis 10/22/2014 15:03' prior: 0!
I am a debugger that provides the ability to step through the execution at the bytecode level!

!GTBytecodeDebugger class methodsFor: 'accessing' stamp: 'AndreiChis 10/22/2014 11:08'!
defaultTitle

	^ 'Bytecode Debugger'! !

!GTBytecodeDebugger class methodsFor: 'tools registry' stamp: 'AndreiChis 10/22/2014 11:13'!
register

	self registerToolsOn: Smalltalk tools.! !

!GTBytecodeDebugger class methodsFor: 'tools registry' stamp: 'AndreiChis 10/22/2014 11:10'!
registerToolsOn: registry

	"Add ourselves to registry. See [Smalltalk tools]" 
	registry registerDebugger: self withRank: 10.! !

!GTBytecodeDebugger class methodsFor: 'accessing' stamp: 'AndreiChis 10/22/2014 14:17'!
sessionClass

	^ GTBytecodeDebuggerSession! !

!GTBytecodeDebugger class methodsFor: 'accessing' stamp: 'AndreiChis 1/14/2015 18:55'!
variablesBrowserClass

	^ GTBytecodeDebuggerVariablesBrowser! !

!GTBytecodeDebugger methodsFor: 'building actions' stamp: 'AndreiChis 10/23/2014 15:16'!
codeActionsPragmas

	^ #( bytecodeDebuggingAction )! !

!GTBytecodeDebugger methodsFor: 'building actions' stamp: 'AndreiChis 10/22/2014 14:07'!
debuggingActionsPragmas

	^ #( bytecodeDebuggingAction )! !

!GTBytecodeDebugger methodsFor: 'building presentations' stamp: 'AndreiChis 1/14/2015 17:58'!
methodBytecodeIn: aComposite forContext: aContext

	^ aComposite tree
		title: 'Bytecode';
		rootsExpanded;
		display: [ aContext method symbolicBytecodes ];
		children: [ :each | each isArray ifTrue: [each] ifFalse: #() ];
		format: [ :each | each printString].! !

!GTBytecodeDebugger methodsFor: 'building presentations' stamp: 'AndreiChis 1/14/2015 17:59'!
methodCodeIn: aComposite forContext: aContext

	(self methodBytecodeIn: aComposite forContext: aContext)
		initialize: [ :bytecodePresentation |
				bytecodePresentation selection: (self session symbolicBytecodeForCurrentPCFrom: self currentContext) ].
	super methodCodeIn: aComposite forContext: aContext.! !

!GTBytecodeDebugger methodsFor: 'building actions' stamp: 'AndreiChis 10/22/2014 14:19'!
stackDebuggingActionsPragmas

	^ #( bytecodeStackDebuggingAction )! !

!FuelOutStackDebugAction class methodsFor: '*GT-BytecodeDebugger' stamp: 'AndreiChis 10/22/2014 15:16'!
gtBytecodeDebuggerActionFor: aDebugger
	<gtBytecodeStackDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			order: 30! !

!RestartDebugAction class methodsFor: '*GT-BytecodeDebugger' stamp: 'AndreiChis 10/22/2014 15:16'!
gtBytecodeDebuggerActionFor: aDebugger
	<gtBytecodeStackDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousRestart! !

!Context methodsFor: '*GT-BytecodeDebugger' stamp: 'AndreiChis 1/14/2015 19:00'!
gtBytecodeDebuggerRetrieveContextValues
	|fieldList displayedTemp context|

	fieldList := OrderedCollection new.
	context := self.
	context == nil ifTrue: [^('_thisContext' -> nil) asOrderedCollection].
	fieldList
		add: ('_self' ->  context receiver);
		add: ('_thisContext' ->  context).
	
	displayedTemp := OrderedCollection new.
	context stackPtr to: 1 by: -1 do: [ :index ||key|
		key := (index-1) asString.
		(index <= context tempNames size) ifTrue: [ 
			key := key, ' [', (context tempNames at: index) asString, ']'.
			displayedTemp add: (context tempNames at: index) ].
		fieldList add: (key -> (context at: index)) ].
	(context tempNames copy asOrderedCollection removeAll: displayedTemp; yourself) ifNotEmptyDo: [ :uninitializedTemps |
		uninitializedTemps do: [ :each |
			fieldList add: ('- [', each, ']') -> nil ] ].
	
	fieldList 
		addAll: ((context receiver class allInstVarNames size to: 1 by: -1) collect: [:index ||name| 
			name := context receiver class instVarNameForIndex: index.
			('rcv', (index-1) asString, ' [', name , ']') -> (context receiver instVarNamed: name) ]).
	
	^ fieldList! !

!ResumeDebugAction class methodsFor: '*GT-BytecodeDebugger' stamp: 'AndreiChis 10/22/2014 15:16'!
gtBytecodeDebuggerActionFor: aDebugger
	<gtBytecodeStackDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousPlay! !

DebugAction subclass: #GTGoAndInspectBytecodeDebugAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-BytecodeDebugger'!

!GTGoAndInspectBytecodeDebugAction commentStamp: 'AndreiChis 10/23/2014 15:20' prior: 0!
GTGoAndInspectBytecodeDebugAction class is a debugging action that opens the currently selected bytecode in the inspector embeded within the debugger.
!

!GTGoAndInspectBytecodeDebugAction class methodsFor: 'as yet unclassified' stamp: 'AndreiChis 10/23/2014 15:18'!
gtBytecodeDebuggerActionFor: aDebugger
	<gtContextBytecodeDebuggingAction>
	
	^ (self forDebugger: aDebugger)! !

!GTGoAndInspectBytecodeDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 10/23/2014 15:15'!
defaultLabel

	^ 'Go and inspect'! !

!GTGoAndInspectBytecodeDebugAction methodsFor: 'actions' stamp: 'AndreiChis 10/23/2014 15:14'!
executeAction
	
	self debugger inspect: self debugger codePresentation selection  
		
! !

!GTGoAndInspectBytecodeDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 10/23/2014 15:15'!
id

	^ GTGoAndInspectBytecodeDebugAction name! !

!GTGoAndInspectBytecodeDebugAction methodsFor: 'as yet unclassified' stamp: 'AndreiChis 10/23/2014 15:18'!
initialize

	super initialize.
	self needsUpdate: false! !

DebugAction subclass: #GTStepIntoBytecodeDebugAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-BytecodeDebugger'!

!GTStepIntoBytecodeDebugAction commentStamp: 'AndreiChis 10/22/2014 15:03' prior: 0!
A GTStepIntoBytecodeDebugAction is debugging action for stepping into bytecode instructions representing message sends.
!

!GTStepIntoBytecodeDebugAction class methodsFor: 'registration' stamp: 'AndreiChis 10/22/2014 15:16'!
gtBytecodeDebuggerActionFor: aDebugger
	<gtBytecodeStackDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousInto! !

!GTStepIntoBytecodeDebugAction methodsFor: 'testing' stamp: 'AndreiChis 10/22/2014 14:21'!
appliesToDebugger: aDebugger

	^ aDebugger session isInterruptedContextPostMortem not! !

!GTStepIntoBytecodeDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 10/23/2014 15:14'!
defaultLabel

	^ 'Step into'! !

!GTStepIntoBytecodeDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 10/22/2014 15:17'!
defaultOrder

	^ 15! !

!GTStepIntoBytecodeDebugAction methodsFor: 'actions' stamp: 'AndreiChis 10/22/2014 14:28'!
executeAction

	self session stepIntoBytecodeInContext: self currentContext! !

!GTStepIntoBytecodeDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 10/22/2014 14:29'!
id

	^ #GTStepIntoBytecodeDebugAction! !

DebugAction subclass: #GTStepOverBytecodeDebugAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-BytecodeDebugger'!

!GTStepOverBytecodeDebugAction commentStamp: 'AndreiChis 10/22/2014 15:04' prior: 0!
A GTStepOverBytecodeDebugAction is a debugging actions that provides the ability to step over a bytecode instruction.
!

!GTStepOverBytecodeDebugAction class methodsFor: 'registration' stamp: 'AndreiChis 10/22/2014 15:16'!
gtBytecodeDebuggerActionFor: aDebugger
	<gtBytecodeStackDebuggingAction>
	
	^ (self forDebugger: aDebugger)
			icon: GLMUIThemeExtraIcons glamorousOver! !

!GTStepOverBytecodeDebugAction methodsFor: 'testing' stamp: 'AndreiChis 10/22/2014 14:26'!
appliesToDebugger: aDebugger

	^ aDebugger session isInterruptedContextPostMortem not! !

!GTStepOverBytecodeDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 10/22/2014 14:26'!
defaultLabel

	^ 'Step over'! !

!GTStepOverBytecodeDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 10/22/2014 15:17'!
defaultOrder

	^ 20! !

!GTStepOverBytecodeDebugAction methodsFor: 'actions' stamp: 'AndreiChis 10/22/2014 14:26'!
executeAction

	self session stepOverBytecodeInContext: self currentContext! !

!GTStepOverBytecodeDebugAction methodsFor: 'accessing' stamp: 'AndreiChis 10/22/2014 14:29'!
id

	^ #GTStepOverBytecodeDebugAction! !

GTDebuggerVariablesBrowser subclass: #GTBytecodeDebuggerVariablesBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-BytecodeDebugger'!

!GTBytecodeDebuggerVariablesBrowser methodsFor: 'as yet unclassified' stamp: 'AndreiChis 1/14/2015 19:09'!
variableTypeOf: aString in: aContext
	|variableName|
	
	variableName := (aString includes: $[)
			ifTrue: [ (aString copyFrom: 1 to: (aString indexOf: $[)-1) trim  ]
			ifFalse: [ aString ].
	^ super 
		variableTypeOf: variableName
		in: aContext! !

!GTBytecodeDebuggerVariablesBrowser methodsFor: 'as yet unclassified' stamp: 'AndreiChis 1/14/2015 19:07'!
variablesFromContext: aContext 

	^ aContext gtBytecodeDebuggerRetrieveContextValues! !

!GTBytecodeDebuggerVariablesBrowser methodsFor: 'as yet unclassified' stamp: 'AndreiChis 1/14/2015 19:02'!
variablesIn: aComposite

	^ (super variablesIn: aComposite)
		sorted: nil! !
